/* -----------------------------------------------------------
                     max7219_dot8x8_4.h

   Header Softwaremodul fuer 4-fach kaskatierte 8x8 DOT-
   Matrixanzeige

   Benoetigte Hardware;

            - 4 Sueck 8x8 LED Matrix mit MAX7219 IC

   MCU   :  STM8S103F3 / STM8S003F3
   Takt  :  externer Takt 16 MHz


   07.04.2022 by R. Seelig
   ----------------------------------------------------------- */

#ifndef indot8x8_r32
  #define indot8x8_r32

  #include <stdlib.h>                          // fuer abs
  #include "stm8s.h"
  #include "stm8_init.h"
  #include "stm8_gpio.h"

  #include "my_printf.h"

  #define font5x7_enable     1                 // 1: Schriftstil verfuegbar, 0: nicht verfuegbar
  #define font8x8_enable     1                 // 1: Schriftstil verfuegbar, 0: nicht verfuegbar

  // Pinzuordnung und Ein- Ausschaltmakros

  #define m7219_din          B2
  #define m7219_clk          B0
  #define m7219_load         B1

  // Pins als Ausgaenge setzen
  #define m7219_init_con()   { m7219_dininit();                      \
                               m7219_clkinit();                      \
                               m7219_loadinit(); }

  #define  printf    my_printf

  typedef char txary[];                   // fuer Strings

  // "Framebuffer" der ein komplettes 32x8 Pixelbitmap aufnehmen kann
  extern uint8_t fbuf[32];

  extern uint8_t fontnr;                  //  0 : 5x7  Font
                                          //  1 : 8x8  Font
  extern uint8_t fontsizex;

  extern uint8_t aktxp;                   // Beinhaltet die aktuelle Position des Textcursors in X-Achse
  extern uint8_t aktyp;                   // dto. fuer die Y-Achse

  extern  const uint8_t font5x7[][5];
  extern  const uint8_t font8x8[][8];

  /* -----------------------------------------------------------
                           Prototypen
     ----------------------------------------------------------- */

  void fb_clr(void);
  void serout(uint8_t data);
  void m7219_send_nl(uint8_t reg, uint8_t data);
  void m7219_send4(uint8_t row, uint8_t *data);
  void m7219_col32(uint8_t reg, uint32_t data);
  void m7219_clrscr();
  void m7219_init4();
  void m7219_setfb(void);
  void m7219_tx_scrollin(int dt, txary *tx);

  void gotoxy(uint8_t x, uint8_t y);
  void setfont(uint8_t fnr);
  void fb_putpixel(int16_t x, int16_t y, uint8_t f);
  void fb_line(int x0, int y0, int x1, int y1, uint8_t color);
  void fb_putchar(uint8_t x, uint8_t y, uint8_t ch);


  // ----------------------------------------------------------------
  // Praeprozessormacros um 2 Stringtexte zur weiteren Verwendung
  // innerhalb des Praeprozessors  zu verknuepfen
  //
  // Bsp.:
  //        #define ionr      A
  //        #define ioport    CONC(PORT, ionr)
  //
  //        ioport wird nun als "PORTA" behandelt
  // ----------------------------------------------------------------
  #define CONC2EXP(a,b)         a ## b
  #define CONC(a,b)             CONC2EXP(a, b)

  #define m7219_dininit()    CONC(P,CONC(m7219_din,_output_init()))
  #define m7219_dinset()     CONC(P,CONC(m7219_din,_set()))
  #define m7219_dinclr()     CONC(P,CONC(m7219_din,_clr()))

  #define m7219_clkinit()    CONC(P,CONC(m7219_clk,_output_init()))
  #define m7219_clkset()     CONC(P,CONC(m7219_clk,_set()))
  #define m7219_clkclr()     CONC(P,CONC(m7219_clk,_clr()))

  #define m7219_loadinit()   CONC(P,CONC(m7219_load,_output_init()))
  #define m7219_loadset()    CONC(P,CONC(m7219_load,_set()))
  #define m7219_loadclr()    CONC(P,CONC(m7219_load,_clr()))


#endif
