/* -------------------------------------------------------
                        ws2812_test.c

     Programm zum Ansprechen von WS2812 LED's

     MCU      :  STM8S103F3
     Hardware :  WS2812b Leuchtdiodenreihe
     Takt     :  interner Takt 16 MHz

     03.05.2017  R. Seelig
   ------------------------------------------------------ */

#include <stdint.h>
#include "stm8s.h"
#include "stm8_init.h"
#include "ws2812.h"

// rgbcol nimmt Farbwerte fuer rot, gruen und blau auf
struct colvalue rgbcol;


// Geschwindigkeiten fuer Aufblenden und Rotieren
#define blendspeed    50
#define rolspeed      45
#define rainspeed     150


// Makros um Schreibarbeit zu sparen, die Funktionen beziehen sich hier fest
// auf <ledbuffer[0]> fuer den Pufferspeicher und auf <rgbcol> fuer die
// Farbvariable

#define setrgbcol(pos)    ( ws_setrgbcol(&ledbuffer[0], pos, &rgbcol) )
#define blendup_left()    ( ws_blendup_left(&ledbuffer[0], ledanz, &rgbcol, blendspeed))
#define blendup_right()   ( ws_blendup_right(&ledbuffer[0], ledanz, &rgbcol, blendspeed))
#define buffer_rl()       ( ws_buffer_rl(&ledbuffer[0], ledanz) )
#define buffer_rr()       ( ws_buffer_rr(&ledbuffer[0], ledanz) )
#define showbuffer()      ( ws_showarray(&ledbuffer[0], ledanz) )
#define clrarray()        ( ws_clrarray(&ledbuffer[0], ledanz) )


// RGB-Farbwerte fuer Regenbogenfarben
const uint8_t rainbowcolor[] =
{
    255,3,3,  80,10,10, 153,102,117,  72,0,224,
  1,159,232, 0,255,000,   255,255,0, 255,219,0,
  255,146,0, 255,073,0,   255,0,255, 160,0,160
};


/* -------------------------------------------------------------------------------------------
                                              M-A-I-N
   ------------------------------------------------------------------------------------------- */
int main(void)
{
  uint8_t  cnt,i;
  uint16_t ci1, ci2;

  sysclock_init(0);                     //  initialisieren mit internem RC-Taktgeber
  ws_init();
  ws_reset();

  clrarray();
  showbuffer();

  while(1)
  {
    // EGA Farben nacheinander links und rechts auf und abblenden

    for (i= 9; i< 16; i++)
    {
      rgbfromega(i+16, &rgbcol);                                       // Farbwert aus EGA-Palette holen
      if (i & 0x01)
        blendup_left();                                                // und links aufblenden
      else
        blendup_right();                                               // rechts aufblenden

      rgbfromega(0, &rgbcol);                                          // Farbe "aus"
      if (i & 0x01)
        blendup_left();                                                // und links ausblenden
      else
        blendup_right();                                               // rechts ausblenden
    }

    // Farben setzen fuer Farbrotation
    rgbfromvalue(0x01, 0x00, 0x00, &rgbcol);
    blendup_left();                                                    // "Hintergrundfarbe" aufblenden

    rgbfromvalue(0x00, 0x00, 0x20, &rgbcol);
    setrgbcol(0);                                                      // Farbe Position 0 setzen
    showbuffer();
    delay(500);

    rgbfromvalue(0x00, 0x00, 0xff, &rgbcol);
    setrgbcol(1);                                                      // dto. Position 1
    showbuffer();
    delay(500);

    rgbfromvalue(0xff, 0xff, 0xff, &rgbcol);
    setrgbcol(2);                                                      // dto. Position 2
    showbuffer();
    delay(500);

    // und rotieren lassen
    for (i= 0; i< 36; i++)
    {
      delay(rolspeed);
      buffer_rl();
      showbuffer();
    }

    // Regenbogenfarben setzen (und beim Setzen anzeigen)
    clrarray();
    for (i= 0; i< ledanz*3; i+= 3)
    {
      rgbcol.r= rainbowcolor[i];
      rgbcol.g= rainbowcolor[i+1];
      rgbcol.b= rainbowcolor[i+2];
      setrgbcol(i/3);
      showbuffer();
      delay(100);
    }

    // Farben rotieren lassen
    for (i= 0; i< 36; i++)
    {
      buffer_rl();
      showbuffer();
      delay(rainspeed);
    }
    // Regenbogenfarben ausblenden
    rgbfromvalue(0x00, 0x00, 0x00, &rgbcol);
    blendup_left();
    clrarray();
    showbuffer();

  }
}

